#!/usr/bin/python
#==============================================================================
#== Title: dwPY_OpenMQTT.py
#== Description:  Simple MQTT/JSON Python Edge Device Client (Python - Paho MQTT Library)
#==
#== Copyright (c)2015 Telit
#== http://www.ilstechnology.com
#== http://www.devicewise.com
#==
#== To Execute:
#== python dwPY_OpenMQTT.py
#==
#== Legal Disclaimer:
#==
#== THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#== INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#== PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#== HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#== OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#== SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#==============================================================================

# Load Python Import Libraries
import sys
import time
import threading
import dwopenmqtt
import Adafruit_BBIO.GPIO as GPIO
import Adafruit_BBIO.ADC as ADC

# dwM2M Developer Community Assigned Tokens...
dwApiURL    = "http://api.devicewise.com/api";
dwApiHost   = "api.devicewise.com";

# dwOpen Member/Thing Assigned Tokens...
dwAppToken  = "????????????????"
dwThingKey  = "BeagleBoneThing";

# =============================================================================
# == Function: userCallBack
# == Purpose:  Handles Method Callbacks
# =============================================================================
def userCallBack( MsgID, MethodName, ParamText ):
  
	print "*** userCallBack... method = ", MethodName
	print "*** ParamText = ", ParamText	
	
	if( MethodName == "setled" ):
		ledValue = dwopen.getFramedText(ParamText, '\"varled\":', '}');
		print "*** Var LED = ", ledValue;
		
		if ledValue == "true":
			print "*** LED ON!!!"
			GPIO.output(led,GPIO.HIGH)		# turn ON LED
		else:
			print "*** LED OFF!!!"
			GPIO.output(led,GPIO.LOW)		# turn OFF LED
			
		dwopen.dwMailboxAck( MsgID, 0, '', '' );		
		
	if( MethodName == "beep" ):
		mSDuration = int(dwopen.getFramedText(ParamText, '\"duration\":', '}'));
		print "*** Beep Duration = ", mSDuration;
		
		if (mSDuration > 2000) or (mSDuration < 0):
			print "*** Invalid Duration Value!!!"
			dwopen.dwMailboxAck( MsgID, -1, 'Invalid Beep Duration - must be within 0 and 2000', '' );	
			
		else:
			print "*** Beep Duration =", mSDuration	

			GPIO.output(buzzer,GPIO.HIGH)		# turn buzzer ON
			time.sleep( mSDuration/1000.0 );
			GPIO.output(buzzer,GPIO.LOW)			# turn buzzer OFF
			
			dwopen.dwMailboxAck( MsgID, 0, '', '' );	

	return;
	

# =============================================================================
# == Setup I/O
# =============================================================================

GPIO.cleanup()
ADC.setup()

# Connect the Grove Rotary Angle Sensor to analog port J3 
potentiometer = "P9_39" 

# Connect the Grove LED to digital port J2
led = "P9_26"
GPIO.setup(led,GPIO.OUT)

# Connect the Grove Buzzer to digital port J3
buzzer = "P9_22"
GPIO.setup(buzzer,GPIO.OUT)

# Full value of the rotary angle is 300 degrees, as per it's specs (0 to 300)
full_angle = 300

# =============================================================================
# == Main Program Logic
# =============================================================================

# Display Banner Information...
print "deviceWISE OpenAPI MQTT/JSON Client Example Program (Python - Paho MQTT Library Base)"
print "(c) ILS Technology, 2015.  All Rights Reserved.\n"

print("Load dwOpenMQTT Library...")
dwopen = dwopenmqtt.dwOpen();
print("dwOpenMQTT Library Loaded...")

#------------------------------------------------
#-- Connect to m2mAIR Cloud Server
#------------------------------------------------
print("Connect to m2mAIR Cloud Server...")
rc = dwopen.mqttConnect( dwApiHost, dwThingKey, dwAppToken, userCallBack );
print "dwOpen API - MQTT Connect Returned... rc = ", rc

print("Waiting 3 Seconds...")
time.sleep( 3 )

print("Begin Publishing Requests to m2mAIR Cloud Server...")

#------------------------------------------------
#-- Log Publish
#------------------------------------------------
print "dwOpen API - Executing Log Publish Request..."
msgText  = "Python MQTT Edge Device... Connected and Active!"

rc = dwopen.dwLogPublish( dwThingKey, msgText );
print "dwOpen API - Log Publish Returned... rc = ", rc

time.sleep( 2 )

#------------------------------------------------
#-- Location Publish
#------------------------------------------------
print "dwOpen API - Sending Location Publish Request..."
#ILST Headquarters - GPS Detail...
locLatitude  = "26.394919"
locLongitude = "-80.112808"
locAltitude  = "11.4"
locSpeed     = "0.0"
locHeading   = "18.8"
locFixType   = "manual"

rc = dwopen.dwLocationPublish( dwThingKey, locLatitude, locLongitude, locAltitude, locSpeed, locHeading, locFixType );
print "dwOpen API - Location Publish Returned... rc = ", rc

time.sleep( 2 )

#------------------------------------------------
#-- Set Attribute
#------------------------------------------------
print "dwOpen API - Executing Set Attribute Request..."

attribKey   = "SerialNumber"
attribValue = "0123456789"

rc = dwopen.dwSetAttribute( dwThingKey, attribKey, attribValue );
print "dwOpen API - Set Attribute Returned... rc = ", rc

time.sleep( 2 )

#------------------------------------------------
#-- Property Publish
#------------------------------------------------

LastSensorValue = -1

for i in range(0,100):

	# Read sensor value from potentiometer
	sensor_value = ADC.read_raw(potentiometer);

	# Did the potentiometer value change?
	if sensor_value <> LastSensorValue:
		# Yes, Publish new rotary value
		print "dwOpen API - Executing Rotary Property Publish..."
		rc = dwopen.dwPropertyPublish( dwThingKey, "Rotary", sensor_value )			
		print "dwOpen API - Rotary Property Publish Returned... rc = ", rc

		print ("sensor_value =", sensor_value) 

		#------------------------------------------------
		#-- Alarm Publish
		#------------------------------------------------

		# Various simulated alarm status...
		if sensor_value == 0:
			alarmState = 0
			alarmMsg   = "Temp - OK"
		else:
			if sensor_value < 200:
				alarmState = 1
				alarmMsg   = "Temp - Warning"		
			else:
				alarmState = 2
				alarmMsg   = "Temp - HIGH"			
					
		
		print "dwOpen API - Executing Alarm Publish Request... ", alarmMsg
		rc = dwopen.dwAlarmPublish( dwThingKey, "alarm1", alarmState, alarmMsg );
		print "dwOpen API - Alarm Publish Returned... rc = ", rc

		# Save our last rotary reading	
		LastSensorValue = sensor_value

	#------------------------------------------------
	#-- Await Incoming Method Execute Requests
	#------------------------------------------------
	
	print "dwOpen API - Awaiting Method Execute Requests..."

	# Wait here for about 10 seconds for cloud methods before proceeding
	for i in range(0,10):
		time.sleep( 1 );
		dwopen.dwCheck();

#------------------------------------------------
#-- Disconnect from m2mAIR Cloud Server
#------------------------------------------------
print("Disconnecting from m2mAIR Cloud Server...")
dwopen.mqttDisconnect()
print("Ok...")

# Exiting the Application...
print( "dwOpen API MQTT/JSON Example Processing Complete, Exiting... Good Bye!" );
sys.exit()


