#!/usr/bin/python
#==============================================================================
#== Title: dwopenmqtt.py
#== Description:  dwOpen API MQTT/JSON Public API Library (Telit Python Library)
#==
#== Copyright (c)2014. ILS Technology, LLC.
#== http://www.ilstechnology.com
#== http://www.devicewise.com
#==
#== To Execute:
#== python dwPY_OpenMQTT.py 
#==
#==============================================================================
#Legal Disclaimer:
#
#Copyright 2014, ILS Technology
#All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#Redistributions of source code must retain the above copyright notice,
#this list of conditions and the following disclaimer.
#
#Redistributions in binary form must reproduce the above copyright
#notice, this list of conditions and the following disclaimer in
#the documentation and/or other materials provided with the distribution.
#
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS``AS
#IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
#TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
#PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
#CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#==============================================================================

##------------------------------------------------------------------------------------------------------------------
## Release Information:
##  V1.0    (JEK, 06/18/2014)   :   Initial release (deviceWISE Open HTTP/JSON Public API - Python)
##  V1.1    (JEK, 11/15/2014)   :   Initial release (deviceWISE Open MQTT/JSON Public API - Python)
##------------------------------------------------------------------------------------------------------------------

import time
import paho.mqtt.client as mqtt

class dwOpen:

      dwApiURL    = "http://api.devicewise.com/api";
      dwApiHost   = "api.devicewise.com";

      myAppToken  = None;
      myThingKey  = None;
      mySessionId = None;
      myCallback = None;
	  
      msgId = 0;
      msgResp = "replyText";
      
      client = mqtt.Client();

      #------------------------------------------------------------------------------
      #-- Function: Callback - on_connect
      #------------------------------------------------------------------------------
      # The callback for when the client receives a CONNACK response from the server.
      def on_connect(self, client, userdata, flags, rc):
          print("<--- Connected, Result Code: "+str(rc))

          # print("---- Subscribing to Topic 'reply'...")
          # Subscribing in on_connect() means that if we lose the connection and
          # reconnect then subscriptions will be renewed.
          # client.subscribe("reply/#")
          # print("---- Subscribe Successful...")

      #------------------------------------------------------------------------------
      #-- Function: Callback - on_message
      #------------------------------------------------------------------------------
      # The callback for when a PUBLISH message is received from the server.
      def on_message(self, client, userdata, msg):
          print("<--- Received Message: '" + str(msg.payload) + "' on topic '" + msg.topic + "' with QoS " + str(msg.qos))
          self.msgResp = str(msg.payload);          
          if( msg.topic == "notify/mailbox_activity" ):  self.dwMailboxCheck(); 
          if( str(msg.payload).find("\"command\":\"method.exec\"") > 0 ):  self.dwProcessMethodExec( str(msg.payload) )
           
      #------------------------------------------------------------------------------
      #-- Function: Callback - on_subscribe
      #------------------------------------------------------------------------------
      # The callback for when a SUBSCRIBE to the server occurs.
      def on_subscribe(self, client, userdata, mid, granted_qos):
          print("<---- Subscribed To: " + str(mid) + " wit QoS " + str( granted_qos ) ); 

      #------------------------------------------------------------------------------
      #-- Function: Callback - on_disconnect
      #------------------------------------------------------------------------------
      # The callback for when a DISCONNECT from the server occurs.
      def on_disconnect(self, client, userdata, rc):
          if rc != 0:
              print("<--- ERROR: Unexpected MQTT Disconnect, rc = "+str(rc))

      #------------------------------------------------------------------------------
      #-- Function: mqttConnect
      #-- Purpose:  Connect to dwOpen MQTT Broker
      #------------------------------------------------------------------------------
      def mqttConnect( self, hostURL, thingKey, applToken, userCallback ):
          rc = -1;
          
          print("Configuring...")
          self.client.reinitialise(client_id=thingKey, clean_session=True, userdata=None)

          self.client.username_pw_set( thingKey, applToken )
          self.client.on_connect = self.on_connect
          self.client.on_message = self.on_message
          self.client.on_subscribe = self.on_subscribe
          self.client.on_disconnect = self.on_disconnect

          self.myCallback = userCallback
		  
          print("Connecting...")
          self.client.connect(hostURL, 1883, 60)

          print("Starting MQTT Processing Loop...")
          self.client.loop_start()

          time.sleep( 2 )

          print("Subscribing to Topic 'reply'...")
          self.client.subscribe("reply/#")
          print("Subscribe Submitted...")
         
          rc = 0;
          
          self.myAppToken = applToken;
          self.myThingKey = thingKey;
          self.dwApiHost  = hostURL;
          
          return rc;
          
      #------------------------------------------------------------------------------
      #-- Function: mqttDisconnect
      #-- Purpose:  Disconnect from dwOpen MQTT Broker
      #------------------------------------------------------------------------------
      def mqttDisconnect( self ):
          rc = -1;

          print("Shutting Down...")
          self.client.unsubscribe("reply");
          time.sleep( 2 )
          # self.client.loop_stop(force=False)
          self.client.disconnect()
          time.sleep( 2 )

          rc = 0;
          
          return rc;

      #------------------------------------------------------------------------------
      #-- Function: mqttCheck
      #-- Purpose:  Check/Service Incoming Messages
      #------------------------------------------------------------------------------
      def mqttCheck( self ):

          self.client.loop();

          return 0;

      #------------------------------------------------------------------------------
      #-- Function: mqttSendReceive
      #-- Purpose:  Send command to dwOpen MQTT Broker (Topic: 'api')
      #------------------------------------------------------------------------------
      def mqttSendReceive( self, jsonText ):

          #print jsonText

          self.msgId = self.msgId + 1;
          msgText = "\""+str(self.msgId).zfill(5)+"\":"; 
          jsonText = jsonText.replace("\"cmd\":", msgText);
          
          #print jsonText

          #print("Publish Message... (Topic: api)")
          self.msgResp = ''
          self.client.publish("api", jsonText)

          # time.sleep( 0.25 ) 
          
          #print("Publish Submitted, Awaiting Response...")
          timeloop = 0;
          while( timeloop < 50 and len( self.msgResp ) < 12 ):  
              time.sleep( 0.1 )
              timeloop = timeloop + 1    

	  if( self.msgResp.find( msgText ) > 0 ):
              msgText = self.msgResp

          return msgText;

      #------------------------------------------------------------------------------
      #-- Function: getElementCount 
      #-- Purpose:  Get Element Count from Raw Response
      #------------------------------------------------------------------------------
      def getElementCount(haystack, needle):
          if( len( needle ) > len ( haystack ) and needle == "" ):
              return 0
          
          count = ( len(haystack) - len( haystack.replace(needle,"")) ) / len( needle )
          return count;

      #------------------------------------------------------------------------------
      #-- Function: getFramedText 
      #-- Purpose:  Get Text Conten Between Two Frame Markers
      #------------------------------------------------------------------------------
      def getFramedText(self, line, string_start, string_end):
          temp = line.split(string_start)[1]

          return temp.split(string_end)[0]
          
      #------------------------------------------------------------------------------
      #-- Function: getElementArray
      #-- Purpose:  Get Element Array from Raw Response
      #------------------------------------------------------------------------------
      def getElementArray(haystack, needle):
          index   = 0
          subsLoc = -1
          lastLoc = -1
          entry   = 0
          final   = []
      
          count = ( len(haystack) - len( haystack.replace(needle,"")) ) / len( needle )
      
          # substring -> string[start:start+length]
      
          while( index < len(haystack) ):
    	         subsLoc = haystack.find(needle, index)

	  if( subsLoc != -1 and subsLoc != lastLoc ):
	      #print("Substring Found At... " + str(subsLoc) )
	      lastLoc  = subsLoc
	      strstart = lastLoc + len(needle) + 1
	      subStack = haystack[strstart:]
	      firstQuote = subStack.find('"')
	      subPart    = subStack[0:firstQuote]
              #print("Substring Text... " + str(subPart) )	                
	      final.append( subPart )

          index = index + 1
    
          return final
   
      #------------------------------------------------------------------------------
      #-- Function: jsonCreateMailboxCheck 
      #-- Purpose:  Generate JSON Mailbox Check Request
      #------------------------------------------------------------------------------
      def jsonCreateMailboxCheck( self ):
          jsonText = "{ \"cmd\": { \"command\": \"mailbox.check\", \"params\": { \"autoComplete\": true, \"limit\": 0 }}}";
          return jsonText;

      #------------------------------------------------------------------------------
      #-- Function: jsonCreateMailboxUpdate
      #-- Purpose:  Generate JSON Mailbox Update Request
      #------------------------------------------------------------------------------
      def jsonCreateMailboxUpdate( self, messageId, updateMsg ):
          jsonText = "{ \"cmd\": { \"command\": \"mailbox.update\", \"params\": {\"id\": \"" + messageId + "\", \"msg\": \"" + updateMsg + "\"}}}";
          return jsonText;

      #------------------------------------------------------------------------------
      #-- Function: jsonCreateMailboxAck
      #-- Purpose:  Generate JSON Mailbox Ack Request
      #------------------------------------------------------------------------------
      def jsonCreateMailboxAck( self, messageId, errorCode, errorMsg, compParams ):
      
          if not compParams:
             jsonText = "{ \"cmd\": { \"command\": \"mailbox.ack\", \"params\": {\"id\": \"" + messageId + "\", \"errorCode\": \"" + str(errorCode) + "\", \"errorMessage\": \"" + errorMsg + "\"}}}";
          else:
             jsonText = "{ \"cmd\": { \"command\": \"mailbox.ack\", \"params\": {\"id\": \"" + messageId + "\", \"errorCode\": \"" + str(errorCode) + "\", \"errorMessage\": \"" + errorMsg + "\", \"params\": " + compParams + " }}}";
          
          return jsonText;
   
      #------------------------------------------------------------------------------
      #-- Function: jsonCreateBindThing 
      #-- Purpose:  Generate JSON Bind Thing Request
      #------------------------------------------------------------------------------
      def jsonCreateBindThing( self, thingKey ):
          jsonText = "{ \"cmd\": { \"command\": \"thing.bind\", \"params\": { \"key\": \"" + thingKey + "\" } } }";
          return jsonText;
   
      #------------------------------------------------------------------------------
      #-- Function: jsonCreateUnbindThing 
      #-- Purpose:  Generate JSON Unbind Thing Request
      #------------------------------------------------------------------------------
      def jsonCreateUnbindThing( self, thingKey ):
          jsonText = "{ \"cmd\": { \"command\": \"thing.unbind\", \"params\": { \"key\": \"" + thingKey + "\" } } }";
          return jsonText;

      #------------------------------------------------------------------------------
      #-- Function: jsonCreateLogPublish
      #-- Purpose:  Generate JSON Log Publish Request
      #------------------------------------------------------------------------------
      def jsonCreateLogPublish( self, thingKey, msgText ):
          jsonText = "{ \"cmd\": { \"command\": \"log.publish\", \"params\": {\"thingKey\": \"" + thingKey + "\", \"msg\": \"" + msgText + "\"}}}";
          return jsonText;

      #------------------------------------------------------------------------------
      #-- Function: jsonCreatePropertyPublish
      #-- Purpose:  Generate JSON Property Publish Request
      #------------------------------------------------------------------------------
      def jsonCreatePropertyPublish( self, thingKey, propKey, propValue ):
          jsonText = "{ \"cmd\": { \"command\": \"property.publish\", \"params\": {\"thingKey\": \"" + thingKey + "\",\"key\": \"" + propKey + "\",\"value\": " + str(round(propValue,2)) + " }}}";
          return jsonText;

      #------------------------------------------------------------------------------
      #-- Function: jsonCreateAlarmPublish
      #-- Purpose:  Generate JSON Alarm Publish Request
      #------------------------------------------------------------------------------
      def jsonCreateAlarmPublish( self, thingKey, alarmKey, alarmState, alarmMsg ):
          jsonText = "{ \"cmd\": { \"command\": \"alarm.publish\", \"params\": {\"thingKey\": \"" + thingKey + "\", \"key\": \"" + alarmKey + "\", \"msg\": \"" + alarmMsg + "\", \"state\": " + str(alarmState) + "}}}";
          return jsonText;
      
      #------------------------------------------------------------------------------
      #-- Function: jsonCreateLocationPublish
      #-- Purpose:  Generate JSON Location Publish Request
      #------------------------------------------------------------------------------
      def jsonCreateLocationPublish( self, thingKey, locLatitude, locLongitude, locAltitude, locSpeed, locHeading, locFixType ):
          jsonText = "{ \"cmd\": { \"command\": \"location.publish\", \"params\": {\"thingKey\": \"" + thingKey + "\", \"lat\": \"" + locLatitude + "\", \"lng\": " + locLongitude + ", \"altitude\": " + locAltitude + ", \"speed\": " + locSpeed + ", \"alti\": " + locHeading + ", \"fixType\": \"" + locFixType + "\" }}}";
          return jsonText;
   
      #------------------------------------------------------------------------------
      #-- Function: jsonCreateSetAttribute
      #-- Purpose:  Generate JSON Set Attribute Request
      #------------------------------------------------------------------------------
      def jsonCreateSetAttribute( self, thingKey, attribKey, attribValue ):
          jsonText = "{ \"cmd\": { \"command\": \"thing.attr.set\", \"params\": {\"thingKey\": \"" + thingKey + "\", \"key\": \"" + attribKey + "\", \"value\": \"" + attribValue + "\"}}}";
          return jsonText;
      
      #------------------------------------------------------------------------------
      #-- Function: jsonCreateGetAttribute
      #-- Purpose:  Generate JSON Get Attribute Request
      #------------------------------------------------------------------------------
      def jsonCreateGetAttribute( self, thingKey, attribKey ):
          jsonText = "{ \"cmd\": { \"command\": \"thing.attr.get\", \"params\": {\"thingKey\": \"" + thingKey + "\", \"key\": \"" + attribKey + "\"}}}";
          return jsonText;

      #------------------------------------------------------------------------------
      #-- Function: jsonCreateThingFind
      #-- Purpose:  Generate JSON Thing Find Request
      #------------------------------------------------------------------------------
      def jsonCreateThingFind( self, thingKey ):
          jsonText = "{ \"cmd\": { \"command\": \"thing.find\", \"params\": {\"thingKey\": \"" + thingKey + "\"}}}";
          return jsonText;

      #------------------------------------------------------------------------------
      #-- Function: dwCheck
      #-- Purpose:  Check/Service Incoming Messages
      #------------------------------------------------------------------------------
      def dwCheck( self ):

          self.client.loop();

          return 0;

      #------------------------------------------------------------------------------
      #-- Function: dwMailboxCheck
      #-- Purpose:  Execute Mailbox Check Request
      #------------------------------------------------------------------------------
      def dwMailboxCheck( self ):
          rc = -1 
      
          print "dwOpen API - Sending Mailbox Check Request...";
          postRequest = self.jsonCreateMailboxCheck();
          print "dwOpen API Mailbox Check JSON Created...\n", postRequest
      
          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse
          
          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;
      
          return rc;

      #------------------------------------------------------------------------------
      #-- Function: dwMailboxUpdate
      #-- Purpose:  Execute Mailbox Update Request
      #------------------------------------------------------------------------------
      def dwMailboxUpdate( self, messageId, updateMsg ):
          rc = -1 
      
          print "dwOpen API - Sending Mailbox Update Request...";
          postRequest = self.jsonCreateMailboxUpdate( messageId, updateMsg );
          print "dwOpen API Mailbox Update JSON Created...\n", postRequest
      
          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse
          
          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;
      
          return rc;

      #------------------------------------------------------------------------------
      #-- Function: dwMailboxAck
      #-- Purpose:  Execute Mailbox Ack Request
      #------------------------------------------------------------------------------
      def dwMailboxAck( self, messageId, errorCode, errorMsg, compParams ):
          rc = -1 
      
          print "dwOpen API - Sending Mailbox Ack Request...";
          postRequest = self.jsonCreateMailboxAck( messageId, errorCode, errorMsg, compParams );
          print "dwOpen API Mailbox Ack JSON Created...\n", postRequest
      
          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse
          
          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;
      
          return rc;
         
      #------------------------------------------------------------------------------
      #-- Function: dwProcessMethodExec
      #-- Purpose:  Execute Method Exec Request
      #------------------------------------------------------------------------------
      def dwProcessMethodExec( self, execRequest ):
          global gCounter
          rc = -1 
      
          print "dwOpen API - Processing Method Exec Request...";
              
          messageId = self.getFramedText(execRequest, '{\"id\":\"', '\",\"thing')
          print "Method.Exec - messageId:  " + messageId

          rc = self.dwMailboxUpdate( messageId, "Processing..." );

          MethodName = self.getFramedText(execRequest, '{\"method\":\"', '\",\"')
          print "Method.Exec - Method Name:  " + MethodName

          execMethod = execRequest[ execRequest.find('\"method\":'): ]
          # print "Method.Exec - execMethod = " + execMethod                  

          ParamText = ''
          if( execMethod.find("\"params\":") > 0 ):
              ParamText = self.getFramedText(execMethod, '\"params\":', ',\"thingDef')
              print "Method.Exec - Parameter Set = " + ParamText

          # Handle Method Processing... Use Global Counter (gCounter)
          
          self.myCallback(messageId, MethodName, ParamText)

          return rc;

      #------------------------------------------------------------------------------
      #-- Function: dwBindThing
      #-- Purpose:  Execute Bind Thing Request
      #------------------------------------------------------------------------------
      def dwBindThing( self, thingKey ):
          rc = -1 
      
          print "dwOpen API - Sending Bind Thing Request...";
          postRequest = self.jsonCreateBindThing( thingKey );
          print "dwOpen API Bind Thing JSON Created...\n", postRequest
      
          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse
          
          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;
      
          return rc;
          
      #------------------------------------------------------------------------------
      #-- Function: dwUnbindThing
      #-- Purpose:  Execute Unbind Thing Request
      #------------------------------------------------------------------------------
      def dwUnbindThing( self, thingKey ):
          rc = -1 

          print "dwOpen API - Sending Unbind Thing Request...";
          postRequest = self.jsonCreateUnbindThing( thingKey );
          print "dwOpen API Unbind Thing JSON Created...\n", postRequest
 
          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse

          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;

          return rc;

      #------------------------------------------------------------------------------
      #-- Function: dwLogPublish
      #-- Purpose:  Execute Log Publish Request
      #------------------------------------------------------------------------------
      def dwLogPublish( self, thingKey, msgText ):
          rc = -1 

          print "dwOpen API - Sending Log Publish Request...";
          postRequest = self.jsonCreateLogPublish( thingKey, msgText );
          print "dwOpen API Log Publish JSON Created...\n", postRequest

          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse

          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;

          return rc;

      #------------------------------------------------------------------------------
      #-- Function: dwSetAttribute
      #-- Purpose:  Execute Set Attribute Request
      #------------------------------------------------------------------------------
      def dwSetAttribute( self, thingKey, attribKey, attribValue ):
          rc = -1 

          print "dwOpen API - Sending Set Attribute Request...";
          postRequest = self.jsonCreateSetAttribute( thingKey, attribKey, attribValue );
          print "dwOpen API Set Attribute JSON Created...\n", postRequest

          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse

          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;

          return rc;

      #------------------------------------------------------------------------------
      #-- Function: dwLocationPublish
      #-- Purpose:  Execute Location Publish Request
      #------------------------------------------------------------------------------
      def dwLocationPublish( self, thingKey, locLatitude, locLongitude, locAltitude, locSpeed, locHeading, locFixType ):
          rc = -1 

          print "dwOpen API - Sending Location Publish Request...";
          postRequest = self.jsonCreateLocationPublish( thingKey, locLatitude, locLongitude, locAltitude, locSpeed, locHeading, locFixType );
          print "dwOpen API Location Publish JSON Created...\n", postRequest
 
          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse

          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;

          return rc;

      #------------------------------------------------------------------------------
      #-- Function: dwAlarmPublish
      #-- Purpose:  Execute Alarm Publish Request
      #------------------------------------------------------------------------------
      def dwAlarmPublish( self, thingKey, alarmKey, alarmState, alarmMsg ):
          rc = -1 

          print "dwOpen API - Sending Alarm Publish Request...";
          postRequest = self.jsonCreateAlarmPublish( thingKey, alarmKey, alarmState, alarmMsg );
          print "dwOpen API Alarm Publish JSON Created...\n", postRequest
 
          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse

          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;

          return rc;

      #------------------------------------------------------------------------------
      #-- Function: dwPropertyPublish
      #-- Purpose:  Execute Property Publish Request
      #------------------------------------------------------------------------------
      def dwPropertyPublish( self, thingKey, propKey, propValue ):
          rc = -1 

          print "dwOpen API - Sending Property Publish Request...";
          postRequest = self.jsonCreatePropertyPublish( thingKey, propKey, propValue );
          print "dwOpen API Property Publish JSON Created...\n", postRequest
 
          # Execute MQTT Send/Receive to dwOpen API Interface
          postResponse = self.mqttSendReceive( postRequest );
          print "dwOpen API MQTT Response Received...\n", postResponse

          if( postResponse.find("success\":true") > 0 ):   
              rc = 0;

          return rc;

